from grammar import Grammar, EOF_CHAR, augment
from lr_automaton import ItemSet, Item

class EarleyParser:
	def __init__(self, grammar):
		self.grammar = grammar	

	def parse(self, input):
		grammar = self.grammar

		# Compute initial itemset
		items   = ItemSet()
		for (left,right) in grammar.productions:
			if left != grammar.start:
				continue
			items.add(Item(0, (left, right), 0))

		# Go through input tokens and parse
		itemsets = []
		for pos in range(0, len(input)):
			token = input[pos]
			items = self._predict(items, pos)
			items = self._complete(itemsets, items)
			# The itemset for the current token in complete
			itemsets.append(items)

			# Compute initial itemset for next token
			items = self._scan(items, token)

		return itemsets

	# Given an itemset and an id number of this itemset return a new itemset
	# with the additional items produced by the "Pricition" step.
	def _predict(self, itemset, itemsetid):
		print "Prediction step not implemented yet"
		return itemset

	# Given an itemset and a token, return a new itemset with the additional
	# items produced by the "Scanning" step.
	def _scan(self, itemset, token):
		print "Scanning step not implemented yet"
		next = ItemSet()
		return next

	# Given the list of previous itemsets a current itemset, add a new itemset
	# with the additional items produced by the "Completion" step.
	def _complete(self, previous_itemsets, itemset):
		print "Completion step not implemented yet"
		return itemset

if __name__ == "__main__":
	G = Grammar(
		terminals    = [ "blau", "gestreift", "glatt", "teuer" ],
		nonterminals = [ "S" ],
		productions  = [
			("S", ["S", "und", "S"]),
			("S", ["S", "oder", "S"]),
			("S", ["blau"]),
			("S", ["gestreift"]),
			("S", ["glatt"]),
			("S", ["teuer"]),
		],
		start = "S",
	)

	print "=== Grammar ==="
	print G

	print "=== Augmented Grammar ==="
	G_prime = augment(G)
	print G_prime

	print "=== Earley Parser ==="
	p = EarleyParser(G_prime)
	result = p.parse(["blau", "und", "gestreift", "oder", "glatt", "und", "teuer", EOF_CHAR])

	for i in range(0, len(result)):
		print "S%s:\n" % i
		print result[i]
