# coding=utf-8
EOF_CHAR = "#"

# Part1: Grammars
def hash_list(list):
	hashed_elements = map(hash, list)
	return reduce(lambda x, y:x^y, hashed_elements, 0)

# A Production is just a python tuple: (nonterminal, list-of-replacements),
# the list-of-replacements is a normal python list of terminals or nonterminals.
# The only reason this is modeled as a separate class is because we need a hash
# function (which python doesn't provide for lists...)
class Production(tuple):
	def __new__(cls, (left, right)):
		return tuple.__new__(cls, (left, right))
	def __hash__(self):
		return hash(self[0]) ^ hash_list(self[1])
	def __str__(self):
		(left,right) = self
		res = "%s →" % left
		if len(right) == 0:
			res += " ε"
		else:
			for e in right:
				res += " %s" % e
		return res

# A Grammar consists of:
#  - (a list of) terminals
#  - (a list of) nonterminals
#  - (a list of) productions
#  - (the) start (nonterminal)
class Grammar:
	def __init__(self, terminals, nonterminals, productions, start):
		self.terminals    = terminals
		self.nonterminals = nonterminals
		if not isinstance(productions[0], Production):
			productions = map(Production, productions)
		self.productions  = productions
		self.start        = start

	def __str__(self):
		res  = "Terminals: %s\n" % self.terminals
		res += "NonTerminals: %s\n" % self.nonterminals
		res += "Productions:\n"
		for production in self.productions:
			res += "\t%s\n" % (production,)
		res += "Start: %s" % self.start
		return res

# Create an augmented grammar
def augment(grammar):
	newStart = grammar.start
	while newStart in grammar.nonterminals or newStart in grammar.terminals:
		newStart += "'"
	return Grammar(
		terminals    = grammar.terminals,
		nonterminals = grammar.nonterminals + [newStart],
		productions  = [ (newStart, [grammar.start]) ] + grammar.productions,
		start        = newStart)
