# coding=utf-8
#
# Copyright (C) 2012 Matthias Braun,
#                    IPD Snelting, Karlsruhe Institute for Technology
from grammar import Production

# An Item consists of:
# - (the) pos(ition of the dot)
# - (a) production
# - (an optional) context
class Item(object):
	def __init__(self, pos, production, context = None):
		self.pos        = pos
		if not isinstance(production, Production):
			production = Production(production)
		self.production = production
		self.context    = context

	# Return element immediately after the dot (or None if dot is behind the
	# last element)
	def after_dot(self):
		(left, right) = self.production
		if self.pos >= len(right):
			return None
		return right[self.pos]

	# Tests if item can be reduced (i.e. the dot is behind the last element)
	def reducible(self):
		return self.pos >= len(self.production[1])

	# returns a new item with the dot moved to the right
	def advanced(self):
		assert self.pos < len(self.production[1])
		return Item(self.pos+1, self.production, self.context)

	def __str__(self):
		(left,right) = self.production
		res = "%s →" % left
		for i in range(0, len(right)+1):
			if i < len(right):
				res += " "
			if i == self.pos:
				res += "⋅"
			if i < len(right):
				res += str(right[i])
		if self.context != None:
			res += "; %s" % self.context
		return res

	def __hash__(self):
		return hash(self.pos) ^ hash(self.production) ^ hash(self.context)
	def __eq__(self, other):
		return self.pos == other.pos and self.production == other.production and self.context == other.context

# A set of items. Can be used like a normal python set, but additionally
# provides a hash function and a nicer __str__ function.
class ItemSet(set):
	def __new__(cls, init = []):
		return set.__new__(cls, init)

	def __hash__(self):
		res = 0
		for e in self:
			res ^= hash(e)
		return res

	def __str__(self):
		res = ""
		for item in self:
			res += "%s\n" % str(item)
		return res


